import xarray as xr
import sasktran2 as sk
import numpy as np
from sasktran2.tests import rtm_comparison_file


def load_scenario(geometry_index: int, atmosphere_index: int, albedo_index: int, test_case: int, numlegendre: int,
                  altitude_spacing: float=500):
    rtm_file = rtm_comparison_file()

    geo = xr.open_dataset(rtm_file, group='geometry_data')
    anc = xr.open_dataset(rtm_file, group='ancillary_data')
    model = xr.open_dataset(rtm_file, group='model_data')

    albedo = float(model['albedo'].isel(albedo=albedo_index))
    albedo = np.ones(len(model['wavelength'])) * albedo

    geo = geo.isel(solar_condition=geometry_index)

    anc = anc.interp(altitude=np.arange(0, 100001, altitude_spacing))

    model = model.isel(solar=geometry_index, composition=atmosphere_index, albedo=albedo_index, test_case=test_case)

    viewing_geo = sk.ViewingGeometry.from_tangent_altitudes(geo.tangent_altitude.values*1000,
                                                            np.zeros_like(geo.tangent_altitude.values),
                                                            7000000 * np.ones_like(geo.tangent_altitude.values))

    geo = sk.Geometry1D(np.cos(float(geo.tangent_sza) * np.pi/180), float(geo.tangent_saa)*np.pi/180, 6371000, anc.altitude.values.astype(float))

    ray_ext = np.outer(anc.rayleigh_scattering_cross_section.values, anc.air_numden.values) * 100 # to /m
    ray_ssa = np.ones_like(ray_ext)

    o3_ext = np.outer(anc.ozone_absorption_cross_section.values, anc.ozone_numden.values) * 100 # to /m
    o3_ssa = np.zeros_like(ray_ext)

    ray_legendre = np.zeros((ray_ext.shape[0], ray_ext.shape[1], numlegendre, 4))

    ray_legendre[:, :, 0, 0] = 1
    ray_legendre[:, :, 2, 0] = 0.5

    ray_legendre[:, :, 2, 1] = 3
    ray_legendre[:, :, 2, 3] = np.sqrt(6) / 2

    if atmosphere_index == 0:
        atmo = sk.Atmosphere.set_manually(3, ray_ext, ray_ssa, ray_legendre, albedo)
    if atmosphere_index == 1:
        ssa = (o3_ssa*o3_ext + ray_ssa * ray_ext) / (o3_ext + ray_ext)

        atmo = sk.Atmosphere.set_manually(3, ray_ext + o3_ext,
                                          ssa,
                                          ray_legendre, albedo)

    if atmosphere_index == 2:
        aero_scat_ext = np.outer(anc.aerosol_scattering_cross_section.values, anc.aerosol_numden.values) * 100
        aero_abs_ext = np.outer(anc.aerosol_absorption_cross_section.values, anc.aerosol_numden.values) * 100

        aero_ssa = aero_scat_ext / (aero_abs_ext + aero_scat_ext)
        aero_ext = aero_scat_ext + aero_abs_ext

        ssa = (o3_ssa*o3_ext + ray_ssa * ray_ext + aero_ssa * aero_ext) / (o3_ext + ray_ext + aero_ext)

        ext = ray_ext + o3_ext + aero_ext

        aero_leg = anc['aerosol_phase_moments'].isel(pmdim1=0, pmdim2=0).values
        aero_leg = aero_leg[:, :numlegendre]
        full_aero_leg = np.zeros((ssa.shape[0], ssa.shape[1], numlegendre, 4))

        full_aero_leg[:, :, :, 0] = aero_leg[:, np.newaxis, :]

        full_aero_leg[np.isnan(full_aero_leg)] = 0
        full_leg = (ray_ssa * ray_ext)[:, :, np.newaxis, np.newaxis] * ray_legendre + (aero_ssa * aero_ext)[:, :, np.newaxis, np.newaxis] * full_aero_leg
        full_leg /= (ssa * ext)[:, :, np.newaxis, np.newaxis]

        atmo = sk.Atmosphere.set_manually(3, ext, ssa, full_leg, albedo)

    config = sk.Config()
    config.nstokes = 3

    return {'atmosphere': atmo, 'geometry': geo, 'config': config, 'result': model, 'viewinggeo': viewing_geo}


if __name__ == "__main__":
    scen = load_scenario(0, 0, 0, 0, 16)